"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
const react_1 = require("react");
const use_ssr_1 = __importDefault(require("use-ssr"));
const FetchContext_1 = __importDefault(require("./FetchContext"));
// Provider ex: useFetch({ url: 'https://url.com' }) -- (overwrites global url)
// TODO - Provider: arg1 = oldGlobalOptions => ({ my: 'new local options'}) (overwrite all global options for this instance of useFetch)
/**
 * Handles all special options.
 * Ex: (+ means not implemented)
 * - url
 * - onMount
 * + timeout
 * + retry - amount of times it will retry
 * + retryDuration - interval at which each retry is done
 */
function useCustomOptions(urlOrOptions, optionsNoURLs) {
    const context = react_1.useContext(FetchContext_1.default);
    const { isServer } = use_ssr_1.default();
    utils_1.invariant(!(utils_1.isObject(urlOrOptions) && utils_1.isObject(optionsNoURLs)), 'You cannot have a 2nd parameter of useFetch when your first argument is an object config.');
    const url = react_1.useMemo(() => {
        if (utils_1.isString(urlOrOptions) && urlOrOptions)
            return urlOrOptions;
        if (utils_1.isObject(urlOrOptions) && !!urlOrOptions.url)
            return urlOrOptions.url;
        if (context.url)
            return context.url;
        return '';
    }, [context.url, urlOrOptions]);
    utils_1.invariant(!!url, 'The first argument of useFetch is required unless you have a global url setup like: <Provider url="https://example.com"></Provider>');
    const onMount = react_1.useMemo(() => {
        if (utils_1.isObject(urlOrOptions))
            return !!urlOrOptions.onMount;
        if (utils_1.isObject(optionsNoURLs))
            return !!optionsNoURLs.onMount;
        return false;
    }, [urlOrOptions, optionsNoURLs]);
    const loading = react_1.useMemo(() => {
        if (isServer)
            return true;
        if (utils_1.isObject(urlOrOptions))
            return !!urlOrOptions.loading || !!urlOrOptions.onMount;
        if (utils_1.isObject(optionsNoURLs))
            return !!optionsNoURLs.loading || !!optionsNoURLs.onMount;
        return false;
    }, [urlOrOptions, optionsNoURLs]);
    const data = react_1.useMemo(() => {
        if (utils_1.isObject(urlOrOptions))
            return urlOrOptions.data;
        if (utils_1.isObject(optionsNoURLs))
            return optionsNoURLs.data;
    }, [urlOrOptions, optionsNoURLs]);
    const path = react_1.useMemo(() => {
        if (utils_1.isObject(urlOrOptions) && urlOrOptions.path)
            return urlOrOptions.path;
        if (utils_1.isObject(optionsNoURLs) && optionsNoURLs.path)
            return optionsNoURLs.path;
        return '';
    }, [urlOrOptions, optionsNoURLs]);
    return { url, onMount, loading, data, path };
}
exports.default = useCustomOptions;
//# sourceMappingURL=useCustomOptions.js.map